/*global angular */
"use strict";

angular.module("Application", []);

(function () {
	var app = angular.module("Application");

	app.factory('$state', ['$levelGenerator', function ($levelGenerator) {
		var level;
		var accuse;
		var email, emailIx;

		function reset() {
			level = $levelGenerator.generate();
			accuse = {
				mode: 'send',
				names: [{}]
			};
			email = [];
			emailIx = 0;
		}

		reset();

		return {
			reset: reset,
			level: function () { return level },
			accuse: function () { return accuse },
			email: function (object) {
				if (typeof object === "object") {
					email.push(object);
				}
				return email[emailIx];
			},
			emailLength: function () { return email.length },
			emailIx: function (increment) {
				if (typeof increment === "number") {
					return emailIx += increment;
				} else {
					return emailIx;
				}
			},
		};
	}]);

	app.factory('$sound', ['$log', function ($log) {
		var type = detectType();
		var cache = {};

		function detectType() {
			var audio = new Audio();
			if (audio.canPlayType("audio/wav") === "probably") {
				return ".wav";
			}
			if (audio.canPlayType("audio/mpeg") === "probably") {
				return ".mp3";
			}
			if (audio.canPlayType("audio/wav") === "maybe") {
				return ".wav";
			}
			if (audio.canPlayType("audio/mpeg") === "maybe") {
				return ".mp3";
			}
			$log.warn("This browser does not appear to support MP3 or WAV audio files. No sounds will play.");
			return undefined;
		}

		function play(name, volume) {
			if (typeof type === "undefined") {
				return;
			}

			var path = "static/" + name + type;
			var audio = (cache[path] || (cache[path] = new Audio(path))).cloneNode(false);

			if (typeof volume === "number") {
				audio.volume = volume;
			}
			audio.play();
		}

		return {
			button: function () {
				play("button", 0.4);
			}
		};
	}]);

	app.factory('$levelGenerator', [function () {
		return {
			generate: function () {
				return {
					startTime: 9,
					eventTime: 22,
					names: ['Paul', 'Sue'],
					activity: 'dump',
				};
			}
		};
	}]);

	app.factory('$emailGenerator', ['$http', '$interpolate', '$sce', function ($http, $interpolate, $sce) {
		var corpus;

		$http.get('data/email.json')
			.success(function (data, status, headers, config) {
				corpus = data;
			});

		function generate(template, parameters) {
			var email = angular.extend(parameters, template);
			email.body = $sce.trustAsHtml(($interpolate(email.body))(email));
			return email;
		}

		return {
			isLoaded: function () { return !!corpus },
			random: function () {
				var parameters = {
					"fromFirst": "Sue",
					"toFirst": "Paul",
					"time": "6am",
				};
				return generate(corpus["business"]["meeting"][0], parameters);
			}
		};
	}]);

	app.controller('GameCtrl', ['$scope', '$interval', '$state', '$emailGenerator', function ($scope, $interval, $state, $emailGenerator) {
		$scope.$state = $state;

		function tick() {
			if ($emailGenerator.isLoaded() && Math.random() < 0.2) {
				var email = $emailGenerator.random();
				$state.email(email);
			}
		}

		$interval(tick, 500);
	}]);

	app.controller('NavCtrl', ['$scope', function ($scope) {
		$scope.mainPanel = 'email';

		$scope.select = function select(name) {
			$scope.mainPanel = name;
		}
	}]);

	app.controller('EmailCtrl', ['$scope', '$state', '$sound', function ($scope, $state, $sound) {
		$scope.$state = $state;
		$scope.$sound = $sound;
	}]);

	app.controller('AccuseCtrl', ['$scope', '$state', '$sound', '$http', function ($scope, $state, $sound, $http) {
		$scope.$state = $state;
		$scope.$sound = $sound;

		$http.get('data/activity.json')
			.success(function (data, status, headers, config) {
				$scope.activity = data;
			});

		$scope.accuse = function accuse() {
			$state.accuse().mode = 'receive';
		}
	}]);

	app.controller('DebugCtrl', ['$scope', '$state', function ($scope, $state) {
		$scope.$state = $state;
	}]);

}());
